/*
  OK G4 OS X11   03/03
  OK G4 OS X  30/10/01
  OK G4       27/01/00   
  OK          04/06/97                                               */
/* compilation LINUX:

 cc -O3 -ffast-math  dfour1.c nrutil.c KSKDV.c nKSKDV.c -lm -o kdv.out              

 cc -O3 -DgraphX11 -ffast-math -I/usr/X11R6/include dfour1.c nrutil.c KSKDV.c nKSKDV.c macXgrapyl2.c -L/usr/X11R6/lib -lX11 -lXext -lm -o kdvX11.out 
 
*/
/*OK 23/05/97                                                        */
/*OK 26/09/95   vers 19 Sept 95                                      */
/*    26/09/95                                                       */
/*                                                                   */
/*    Resolution de SOLITON LAMB        p 173                        */
/*                           2                2                      */
/*    du   du  3       du   d   du          du                       */
/*    -- + --+ - eps u -- + --  ---  = visc ---                      */
/*    dt   dx  2       dx   6     3           2                      */
/*                              dx          dx                       */
/* biblio Frish She Thual JFM 1986 pp221 240                         */
/*        Hyman NICOLAENKO  Physica 18D 1986 113-126                 */
/*        Manneville Physics Letters V84A n3 (1981)                  */
/*                                                                   */
#include <stdio.h>
#include <stdlib.h>
#include "math.h"
#include "rgraph2.h"  
#include "param.h"

#ifdef graphX11
//dans ce cas affichage X11
#else
#define batch
#endif

#define pi 3.14159 
 #ifdef powerc
 #include <sioux.h>
 #else
#define pi 3.14159 
 #endif
/*#include "unix.h"*/
double dx,dt,t,Lx,UN;
double epsilon,delta2,visc,sigma;
double* u=NULL;
double* TFu=NULL;
double* TFdu=NULL;
double* TFdu2=NULL;
double* uo=NULL;
double* Espect=NULL; 
double* e=NULL;
double eit;
double Pi;
FILE *g;
FILE *fs;
int nx,NLin;
float x,x0,xbas,ybas,xhaut,yhaut;
int npix,npiy,af;
void energie(int it,int ittmax);
 void tracsauv(int it);
 void axes(float *xbas,float *x0,int *nx,float *dx,
    float *ybas,float *xhaut,float *yhaut,int *npix,int *npiy);
      
/*void MainEvent(void);*/
/*                                                                   */
int main(void)
{
int i,it,itt,itmax,ittmax;
int nbt;
int ilarge;
float epsilarge;
float fdx,fdt,fepsilon;
double omega;
double u0;
/* Place the windows top left corner at (5,40).      */
 #ifdef powerc
 SIOUXSettings.toppixel =40;
 SIOUXSettings.leftpixel =10; 
 SIOUXSettings.rows = 10;
 SIOUXSettings.columns = 80; 
 #endif

/* ouverture des fichiers de sauvegarde  */
g = fopen("F.IN", "r");
printf("Bench                                                       \n");
printf("PPC60 3 it en 16s PPC66   11s                               \n");
printf("1 it PPC60=5.3 PPC66=3.6 355=3.2 370=2.2 (1/2)3BT= 2.5      \n");
printf(" 3BT=11s   Cyrix133/P166=13s LinuxPPC66MHz=17s MWCPPC66=33s \n");
printf("PII_200=3s  Cyrix166=4.3      LinuxPPC66MHz=15s             \n");
printf("                                                            \n");
printf("PIII_550=10.7s  ppcG4(601)=17,8s G4(604)=14.3s PPC60=178s   \n");
printf("G4_400 1s G3_500 0.9s                                       \n");       
printf("                                                            \n");       
printf("                                               \n");
printf("                                               \n");
printf(" Resolution des equations (c.f. LAMB)          \n");
printf("                                               \n");
printf("                          2                2   \n");
printf("   du   du  3       du   d   du          du    \n");
printf("   -- + --+ - eps u -- + --  ---  = visc ---   \n");
printf("   dt   dx  2       dx   6     3           2   \n");
printf("                             dx          dx    \n");
printf("                                            \n");
printf("   Par decomposition spectrale              \n");
printf("     ___                                    \n");
printf("^    \\          I k (2 Pi j/N)             \n");
printf("u  = /    u    e                            \n");
printf(" k   ---    j                               \n");
printf("                                            \n");
printf("                                            \n");
printf("                                            \n");
printf("     ^ n+1  ^ n    2      3         2 ^ n+1 \n");
printf("donc u    = u  - (d /6 I k  - visc k )u     \n"); 
printf("                                -1     ^  n \n");
printf("                     -dt(TF(u TF  (-Ik)u))  \n"); 
printf("                                            \n");
printf("                                            \n");
printf("                                            \n");
printf("                                            \n");
printf("    autre methode                           \n");
printf("   ^     ^   ^                              \n");
printf("  du/dt=Lu + s                              \n");
printf("        L partie lineaire                   \n");
printf("              ^                             \n");
printf("              s  terme source non lineaire  \n");
printf("                                            \n");
printf("    est resolu a la main!                   \n");
printf("                                            \n");
printf("                            Ldt             \n");
printf("     ^         Ldt^     ^  e    - 1         \n");
printf("     u(t+dt)= e   u(t) +s ---------         \n");
printf("                                L           \n");
printf(" [return]                                   \n");
//getchar(); 
printf(" fichier d'entree F.IN                      \n");
/* lecture du fichier donnees */
fscanf(g,"dt=%f                                               \n",&fdt);
dt=fdt;
printf("pas de temps   dt=%f                                    \n",dt);
fscanf(g,"itmax=%i                                          \n",&itmax);
fscanf(g,"ittmax=%i                                        \n",&ittmax);
printf("affichage toutes les %i iter, soit dt=%f  \n",ittmax,dt*ittmax);
printf("itmax=%d => t max=itmax*ittmax*dt=%7f\n",itmax,itmax*ittmax*dt);
fscanf(g,"nx=%i                                        \n",&nx);

x0=0;
Pi=atan(1)*4;
/*Lx=3*Pi; PI */
printf("nbre de points: nx=%d,                            ",nx);
fscanf(g,"Lx=%lg                                       \n",&Lx);
dx=2*Lx/nx;
printf("pas d'espace: dx=%f,                              ",dx);
printf(" %g < domaine < %g =%g         \n",-nx*dx/2,nx*dx/2,Lx);
fscanf(g,"UN=%lg                                       \n",&UN);
printf("si %lf=1 dans rep. labo. sinon repere de l'onde \n",UN);
fscanf(g,"epsilon=%lf                             \n",&epsilon);
printf("parametre non lineaire epsilon=%lf         \n",epsilon);
fscanf(g,"visc=%lg                                   \n",&visc);
printf("            parametre visqueux=%f             \n",visc);
visc=visc*pow(Pi/Lx,2);
  /* (Pi/Lx) pour etre entre -L et L*/
fscanf(g,"delta2=%lg                               \n",&delta2);
printf("               delta au carre=%lg           \n",delta2);
delta2=delta2*pow(Pi/Lx,3);
  /* (Pi/Lx) pour etre entre -L et L*/
fscanf(g,"sigma=%f                               \n",&fepsilon);
sigma=(double)fepsilon;
printf("             tension de surf=%g              \n",sigma);
   
 
 
NLin=1;
 
fscanf(g,"ilarge=%d \n",&ilarge);
printf("si (ilarge=%d)=0 u(t=0)=1/ch2 (soliton)en C.I. \n",ilarge);
printf("si (ilarge=%d)=1 toutes les frequences en C.I. \n",ilarge);
printf("si (ilarge=%d)=2....                           \n",ilarge);
printf("si (ilarge=%d)=3....                           \n",ilarge);
printf("si (ilarge=%d)=4  1+cos                        \n",ilarge);
printf("si (ilarge=%d)=5 cos(x) Zabusky 65             \n",ilarge);
printf("si (ilarge=%d)=6  cas du mascaret              \n",ilarge);
epsilarge=.00000001;
fscanf(g,"epsilarge=%f \n",&epsilarge);
printf("epsilarge=%f amplitude de la perturbe \n",epsilarge);
 fscanf(g,"omega=%lg \n",&omega);
 printf(" omega=%lg \n", omega);
dt=dt ;
printf("pas de temps   dt=%f\n",dt);
printf("affichage toutes les %i iter, soit dt=%f \n",ittmax,dt*ittmax);
printf("itmax=%d     t max=itmax*ittmax*dt= %7f  \n",
  itmax,itmax*ittmax*dt);
fclose(g);
 
//fs= fopen("Fs.OUT","w");
g = fopen("Fxh.OUT","w"); fclose(g);
printf(" fichier de sortie F.OUT    et Fxh.OUT               \n");
printf(" [return]                                   \n");
//getchar();  

/* fin des lectures */
/* declaration */
nbt=nx+15;
u =(double*)calloc(nbt,sizeof(double));
uo=(double*)calloc(nbt,sizeof(double));
Espect=(double*)calloc(nbt,sizeof(double));
nbt=2*nx+30;
TFu  =(double*)calloc(nbt,sizeof(double));
TFdu =(double*)calloc(nbt,sizeof(double));
TFdu2=(double*)calloc(nbt,sizeof(double));
nbt=itmax+10;
e=(double*)calloc(nbt,sizeof(double));
/* fin declaration */
/* &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
/*            init graphique/ trace des axes           */   
/* pour QT */
af=99; af=1;   
if(af==99){
printf("ATTENTION FILM!!! SILENCE ON TOURNE!\n");
getchar();}


#ifdef batch
#else          
initgraphique(af);
fdx=(float)dx;
axes(&xbas,&x0,&nx,&fdx,&ybas,&xhaut,&yhaut,&npix,&npiy); 
#endif
/* &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
printf(".............................................\n");
/* Initialisation conditions initiales */
 t=0;
  for (i=0; i<=nx; i++)
  {
  /*RAZ ou CI */
   x=-Lx +((i)*dx )   ;
 
if(ilarge==0){ 
   // u[i]=2*exp( -4*(x-3.14)*(x-3.14)); 
  // u[i]=epsilarge*cos(omega*x);*/
  
   u0=2;
   u[i]= u0*pow(1/cosh(sqrt(3)*sqrt(u0)*(-(t*(3 + 3*epsilon*u0/2))/3 + (x+3*Lx/4))/2),2);
  }
   /* on met toutes les frequences */
if(ilarge==1){ 
   u[i]=0;
   itt=nx;
  for(it=1;it<itt;it++){
   u[i]=u[i]+epsilarge*(cos( it*x)+sin(it*x));} }
   
if(ilarge==2){ 
   u[i]=0.75*sin(2*pi*(x-sqrt(tanh(2*pi*sqrt(delta2))/(2*pi*sqrt(delta2)))));;}  
 
 
if(ilarge==3){ 
   u[i]=1/ pow((1-epsilarge*epsilarge),.5)-
   2 *pow((1-epsilarge*epsilarge),.5)/(1-epsilarge*cos(x));   
   }  
   
if(ilarge==4){ 
   u[i]=1+(cos(2*Pi*x/Lx));}

   
if(ilarge==5){ 
     /* Zabusky 65 */
   u[i]= (cos(x ));
   } 
   
if(ilarge==6){ 
     /*  ressaut mars 2003  */
    // if (i<nx/2){u[i]=1;} else{u[i]=0;}
     u[i]=1-tanh(x/2);
   }       
   
   uo[i]=u[i];}
/* fin C.I.                                           */








for (it=0; it<=itmax; it++)
{
/* ================================================================*/
for (itt=1; itt<=ittmax; itt++)
{
/* avancee */  
  next( );
  t=t+dt; 
//  MainEvent();  
}
/* ================================================================*/ 
  printf(" t= %7f       [return]",t);

#ifdef batch
#else   
  reffecran(); 
#endif 
 
 
 
 
 
 
 
 
 
 
 
/*  les traces et les sauvegardes         */  

   tracsauv(it);
  
  //getchar();
}/* fin boucle temps */
//fclose(fs);
}
 

/*---------------------------------------------------------*/
/*---------------------------------------------------------*/
 

/*---------------------------------------------------------*/
void tracsauv(int it)
{
FILE *f;
double um;
int i;
extern int nx;
extern double *u,epsilon,UN;
extern double dx,dt,t;
extern float x,x0,xbas,ybas,xhaut,yhaut;
extern int npix,npiy;
float xp,yp;
int icoul;
double u0;

  f = fopen("F.OUT","w");
    x0=0;
    xp=18;
    yp=0;
   icoul=80;
   /* trace l'axe 0 */
#ifdef batch
#else 
metligne(xbas,ybas,xhaut,yhaut,npix,npiy,x0,yp,xp,yp,icoul);
#endif
   
   /* on calcule la moyenne */
   um=0;
   for (i=1; i<=nx; i++)
   {um=um+u[i];}
   um=um/nx;
   for (i=1; i<=nx; i++)
   {
   /* on commence par les traces et les sauvegardes         */
   x=(i)*dx+x0;
   xp=x;
   yp=(u[i]-0) ;
   
   icoul=80-3*i*2;icoul=1;
#ifdef batch
#else    
   metpoint(xbas,ybas,xhaut,yhaut,npix,npiy,xp,yp,icoul);
#endif   
   u0=2; 
    x=-Lx +((i)*dx );
    fprintf(f," %f %f  \n",x,u[i]);
   yp= u0*pow(1/cosh(sqrt(3)*sqrt(u0)*(-(t*(3*(UN) + 3*epsilon*u0/2))/3 + (x+3*Lx/4  ))/2),2);
    
   icoul=150;
#ifdef batch
#else    
   metpoint(xbas,ybas,xhaut,yhaut,npix,npiy,xp,0*(yp-u[i]),icoul);
#endif
   }
  fclose(f);
  
   f = fopen("Fxh.OUT","a");
     for (i=1; i<=nx; i++)
   {
   /* on commence par les traces et les sauvegardes         */
   x=(i)*dx-Lx;
   xp=x; 
   fprintf(f," %f %f \n",xp,u[i]);}
   fprintf(f,"\n\n");
    fclose(f);
    /* fichier pour GNUPLOT */
   f= fopen("mov.gnu","w");    
   fprintf(f,"print k                          \n");
   fprintf(f,"p'Fxh.OUT'i k u 1:(1+$2/10)t'h(x,t)'w l  \n");
   fprintf(f,"k=k+1                            \n");
   fprintf(f,"if(k<%d) reread               \n",it);printf("%d\n",it);
   fclose(f);
 /* favorable au multi tache!!!!!!!*/
}


#ifdef batch
#else 
/*---------------------------------------------------------*/
void axes(float *xbas,float *x0,int *nx,float *dx,
      float *ybas,float *xhaut,float *yhaut,int *npix,int *npiy)
{
/* trace des axes */
int i;
float xp,yp;
*xbas=*x0-1;
*ybas=-2;
*xhaut=*x0+*nx*(*dx) ;
*yhaut=3;
*npix= 350;
*npiy= 250;
for (i=0;i<*npix; i++)
{xp=i* *xhaut * 2 * 1.f / *npix;yp=0;
 metpoint(*xbas,*ybas,*xhaut,*yhaut,*npix,*npiy,xp,yp,i);
 xp=0;yp=i*(*dx);
 metpoint(*xbas,*ybas,*xhaut,*yhaut,*npix,*npiy,xp,yp,i);
 }
 }
#endif 
 

